@@ -3,6 +3,7 @@ import pytest
 import os
 from rt_utils import RTStructBuilder
 from rt_utils.utils import SOPClassUID
+from rt_utils import image_helper
 from pydicom.dataset import validate_file_meta
 import numpy as np
 
@@ -121,7 +122,7 @@ def test_loaded_mask_iou(new_rtstruct: RTStruct):
     mask[50:100, 50:100, 0] = 1
     mask[60:150, 40:120, 0] = 1
 
-    IOU_threshold = 0.95 # Expected accuracy
+    IOU_threshold = 1.0 # Expected accuracy
     run_mask_iou_test(new_rtstruct, mask, IOU_threshold)
 
 
@@ -131,7 +132,7 @@ def test_mask_with_holes_iou(new_rtstruct: RTStruct):
     mask[50:100, 50:100, 0] = 1
     mask[65:85, 65:85, 0] = 0
 
-    IOU_threshold = 0.85 # Expect lower accuracy since holes lose information
+    IOU_threshold = 0.95 # Expect lower accuracy since holes lose information
     run_mask_iou_test(new_rtstruct, mask, IOU_threshold)
 
 
@@ -141,7 +142,7 @@ def test_pin_hole_iou(new_rtstruct: RTStruct):
     mask[50:100, 50:100, 0] = 1
     mask[65:85, 65:85, 0] = 0
 
-    IOU_threshold = 0.85 # Expect lower accuracy holes lose information
+    IOU_threshold = 0.95 # Expect lower accuracy holes lose information
     run_mask_iou_test(new_rtstruct, mask, IOU_threshold, use_pin_hole=True)
     
 def test_no_approximation_iou(new_rtstruct: RTStruct):
@@ -149,7 +150,7 @@ def test_no_approximation_iou(new_rtstruct: RTStruct):
     mask[50:100, 50:100, 0] = 1
     mask[60:150, 40:120, 0] = 1
 
-    IOU_threshold = 0.95 # Expected accuracy
+    IOU_threshold = 1.0 # Expected accuracy
     run_mask_iou_test(new_rtstruct, mask, IOU_threshold, approximate_contours=False)
 
 def test_contour_data_sizes(new_rtstruct: RTStruct):
@@ -164,6 +165,22 @@ def test_contour_data_sizes(new_rtstruct: RTStruct):
     # Then using approximation leads to less data within the contour data
     assert get_data_len_by_index(new_rtstruct, 0) < get_data_len_by_index(new_rtstruct, 1)
 
+def test_nonstandard_image_orientation(oriented_rtstruct: RTStruct):
+    mask = get_empty_mask(oriented_rtstruct)
+    mask[10:70, 5:15, 1] = 1
+    mask[60:70, 5:40, 1] = 1
+
+    IOU_threshold = 1.0 # Expected accuracy
+    run_mask_iou_test(oriented_rtstruct, mask, IOU_threshold)
+
+def test_one_slice_image(one_slice_rtstruct: RTStruct):
+    mask = get_empty_mask(one_slice_rtstruct)
+    mask[10:70, 5:15, 0] = 1
+    mask[60:70, 5:40, 0] = 1
+
+    IOU_threshold = 1.0 # Expected accuracy
+    run_mask_iou_test(one_slice_rtstruct, mask, IOU_threshold)
+
 def get_data_len_by_index(rt_struct: RTStruct, i: int):
     return len(rt_struct.ds.ROIContourSequence[i].ContourSequence[0].ContourData)
 
@@ -176,7 +193,7 @@ def run_mask_iou_test(rtstruct:RTStruct, mask, IOU_threshold, use_pin_hole=False
     # Use IOU to test accuracy of loaded mask
     numerator = np.logical_and(mask, loaded_mask)
     denominator = np.logical_or(mask, loaded_mask)
-    IOU =  np.sum(numerator) / np.sum(denominator) 
+    IOU = np.sum(numerator) / np.sum(denominator)
     assert IOU >= IOU_threshold
 
 
